/****************************************************************************
** Demonstration of creating a rain/waterfall like display                 **
**  by Steven H Don                                                        **
**                                                                         **
** For questions, feel free to e-mail me.                                  **
**                                                                         **
**    shd@earthling.net                                                    **
**    http://shd.cjb.net                                                   **
**                                                                         **
****************************************************************************/
#include <dos.h>
#include <stdlib.h>

#ifdef __DJGPP
  #include <sys\nearptr.h>
  #include <sys\movedata.h>
  char *screen;
  void randomize()
  {
    srandom(time());
  }
#else
  char far *screen;
#endif

//This sets the display to VGA 320x200 in 256 colours
void VGAScreen ()
{
  union REGS r;
  r.h.ah = 0;
  r.h.al = 0x13;
  int86(0x10, &r, &r);
}

//This resets the display to text mode
void TextScreen ()
{
  union REGS r;
  r.h.ah = 0;
  r.h.al = 0x3;
  int86(0x10, &r, &r);
}

//This sets a DAC register to a specific Red Green Blue-value
void SetDAC (unsigned char DAC, unsigned char R, unsigned char G, unsigned char B)
{
  outportb (0x3C8, DAC);
  outportb (0x3C9, R);
  outportb (0x3C9, G);
  outportb (0x3C9, B);
}

//Sets a pixel in the buffer to a specific colour
void PutPixel (unsigned int x, unsigned int y, unsigned char c)
{
  if ((x < 320) && (y < 200)) {
    screen [(y << 8) + (y << 6) + x] = c;
  }
}


//This sets up a nice blue palette - am I obsessed with blue?
void BluePalette ()
{
  int DAC;

  for (DAC = 1; DAC < 256; DAC++)
    SetDAC (DAC,
            DAC >> 3,
            DAC >> 3,
            32 + (DAC >> 3));
}

//This rotates the palette
void RotatePal ()
{
  int DAC, OffSet;

  do {
    //Set up the new colours
    for (DAC = 0; DAC < 256; DAC++)
      //don't change colour 0 or the screen border would flash
      if ((DAC + OffSet) & 0xFF)
        SetDAC ((DAC + OffSet) & 0xFF, DAC >> 3, DAC >> 3, 32 + (DAC >> 3));
    OffSet++;
  } while (!kbhit());
}

//This makes the pattern used for the "rain"
void MakeDrip ()
{
  unsigned int x, y, r, s;

  for (x = 0; x < 320; x++) {
    //For each column on screen, choose a different starting point...
    #ifdef __DJGPP
      r = (random ()%255) << 8;
      //...and speed
      s = (128 + (random ()%128)) << 3;
    #else
      r = random (255) << 8;
      //...and speed
      s = (128 + random (128)) << 3;
    #endif
    for (y = 0; y < 200; y++) {
      //Set each pixel on the column, don't use colour 0
      if (!(r & 0xff00))
        PutPixel (x, y, 1);
      else
        PutPixel (x, y, r >> 8);
      //Move on to the next colour
      r += s;
    }
  }
}

void main ()
{
  //Set up a pointer to the vga memory at A000:0000
  #ifdef __DJGPP
    __djgpp_nearptr_enable ();
    screen = (char *) (__djgpp_conventional_base + 0xA0000);
  #else
    screen = MK_FP (0xA000,0);
  #endif

  //Switch to graphics mode
  VGAScreen ();

  //Set up a nice, blue palette
  BluePalette ();

  //Initialize random number generator
  randomize ();

  //Make the pattern
  MakeDrip ();

  //Rotate the palette
  RotatePal ();

  //Switch back to text mode
  TextScreen ();
}